//=============================================================================
//  mesh conversions
//    from files with format :
//        .msh : usually generated by gmsh
//                     (http://www.geuz.org/gmsh/)
//        .mel : used by melina finite element code
//                     (http://perso.univ-rennes1.fr/daniel.martin/melina/)
//    to files with format :
//        .msh : usually generated by gmsh
//                     (http://www.geuz.org/gmsh/)
//        .mel : used by melina finite element code
//                     (http://perso.univ-rennes1.fr/daniel.martin/melina/)
//        .vtk : used by the 3D visualization tool paraview
//                     (http://www.paraview.org/)
//        .amdba : old mesh format used by some applications
//===============================================================================


#include "mesh.h"

using namespace std;

// global variables
extern map<uint,element_type> element_type_list; // load in init()
extern string context; // contains the chain of called functions (error utility)
bool verbose, debug;
ofstream ofdebug;


// main
int main(int argc, char** argv) {
    context="main";

    // initialisation (def of elements, ...)
    init();
    // arguments analysis
    if (argc<3 || ((argc>=3) && ((argv[argc-2][0]=='-') || (argv[argc-1][0]=='-')))) {
        cout << "help : convmesh -h" << endl;
        exit(0);
    }
    
    vector<string> listargstmp, listargs;
    
    for (int i=1;i<argc-2;i++) {
    	listargstmp.push_back(argv[i]);
    }
    int listargssize=listargstmp.size();
    

    verbose=false;
    debug=false;
    language lg=francais;
    string f_in=string(argv[argc-2]);
    string f_out=string(argv[argc-1]);
    
    
    if (listargssize!=0 ) {
    	for (int i=0;i<listargssize;i++) {
    		string arg=listargstmp[i];
    		int argsize=arg.size();

    		if ((arg[0]=='-') && (arg[1]!='-') && (argsize>2)) {
    			for (int j=1;j<argsize;j++) {
    				stringstream ss;
    				ss << "-" << arg[j];
    				listargs.push_back(ss.str());
    			}
    		} else {
    			if (arg[0]=='-') {
    				listargs.push_back(arg);
    			}
    		}
    	}
    
    	for (int i=0;i<int(listargs.size());i++) {
    		string arg=listargs[i];
    		
    		if ((arg=="-h") || (arg=="--help")) {
    			cout << "Syntax :" << endl;
    			cout << "         convmesh [options] filemesh_in.ext1 filemesh_out.ext2" << endl;
        		cout << "         convmesh [options] filemesh_in.ext1 ext2" << endl;
        		cout << "            ext1 = msh | mel ; ext2 = msh | mel | vtk | amdba" << endl;
        		cout << endl;
        		cout << "Options :" << endl;
        		cout << "            -c, --credits         print convmesh credits." << endl;
        		cout << "            -d, --debug           enable debug mode." << endl;
        		cout << "            -e, --english         use english as language." << endl;
        		cout << "            -f, --french          use french as language." << endl;
        		cout << "                                  (default)" << endl;
        		cout << "            -h, --help            print this help." << endl;
        		cout << "            -v, --verbose         enable verbose mode." << endl;
        		cout << "                                  (not in default)" << endl;
        		exit(0);
    		}
    		
    		if ((arg=="-c") || (arg=="--credits")) {
    			cout << "Credits :" << endl;
    			cout << "            E. LUNEVILLE, N. KIELBASIEWICZ, U.M.A/POems, ENSTA_ParisTech" << endl;
        		exit(0);
    		}
    		
    		if ((arg=="-d") || (arg=="--debug")) {
    			debug=true;
    		}
    		
    		if ((arg=="-e") || (arg=="--english")) {
    			lg=english;
    		}
    		
    		if ((arg=="-f") || (arg=="--french")) {
    			lg=francais;
    		}
    		
    		if ((arg=="-v") || (arg=="--verbose")) {
    			verbose=true;
    		}
    	}
    }
	
	if (debug) {
    	ofdebug.open("debug.dat");
	}
	
    // define language and initialize messages
    init_messages(lg);

    // file analysis
    size_t p=f_in.find_last_of('.',f_in.size());

    if (p==string::npos) {
        message_event("err_filein",msg_data(f_in));
    }

    string ext_in=f_in.substr(p+1,f_in.size());

    if (ext_in!="mel" && ext_in!="msh") {
        message_event("err_filein",msg_data(f_in));
    }

    size_t q=f_out.find_last_of('.',f_out.size());
    string ext_out=f_out;

    if (q==string::npos) {
        f_out=f_in.substr(0,p)+"."+ext_out;

    } else {
        ext_out=f_out.substr(q+1,f_in.size());
    }

    if (ext_out!="mel" && ext_out!="msh" && ext_out!="vtk" && ext_out!="amdba") {
        message_event("err_fileout");
    }

    if (ext_out==ext_in) {
        message_event("err_inout");
    }

    // new mesh from file
    mesh M(f_in.c_str());
	
    if (debug) {
    	M.print(ofdebug,true);
    }
    
    // export mesh
    if (ext_out=="mel") {
        M.saveto_mel(f_out.c_str(),false);
    }

    if (ext_out=="msh") {
        M.saveto_msh(f_out.c_str());
    }

    if (ext_out=="vtk") {
        M.saveto_vtk(f_out.c_str());
    }

    if (ext_out=="amdba") {
        M.saveto_amdba(f_out.c_str());
    }

	if (debug) {
    	ofdebug.close();
	}
	
    return 0;
}
